/*
This file is part of libmspframework
Copyright © 2006  Mikko Rasa, Mikkosoft Productions
Distributed under the LGPL
*/
#include <sys/time.h>
#include <errno.h>
#include "semaphore.h"
#include "../time/timestamp.h"
#include "../time/units.h"
#include "../time/utils.h"

namespace Msp {

int Semaphore::wait(const Time::TimeDelta &d)
{
#ifndef WIN32
	Time::TimeStamp ts=Time::now()+d;

	timespec timeout;
	timeout.tv_sec=ts.raw()/1000000;
	timeout.tv_nsec=(ts.raw()%1000000)*1000;

	MutexLock l(mutex);
	int r=pthread_cond_timedwait(&sem, &mutex.mutex, &timeout);
	if(r==ETIMEDOUT)
		return 1;
	else if(r)
		return -1;
	return 0;
#else
	return WaitForSingleObject(sem, (DWORD)(d/Time::usec))==WAIT_OBJECT_0;
#endif
}

} // namespace Msp
